﻿using System;
using System.IO;
using UIKit;
using Xamarin.Forms;
using System.Drawing;
using Foundation;
using static Xamarin.Forms.Internals.GIFBitmap;

[assembly: Dependency(typeof(LayoutDesignerSDK_sample.iOS.PrintSample))]
namespace LayoutDesignerSDK_sample.iOS
{
    //Native side class.
    public class PrintSample : IPrintSample
    {
        private LayoutDesignerSDK.LibApi api = new LayoutDesignerSDK.LibApi();
        private string SdkVersion = @"";
        private string EngVersion = @"";

        //SDK initialization.
        public void init_SDK()
        {
            api.InitSdk();
            api.GetApiVersion(ref SdkVersion, ref EngVersion);
            api.AlternativeFontNameSet(@"Hiragino Sans");
            Console.WriteLine(@"init_SDK Calling");
        }

        //Printing layout files.
        public int PrintFile(string file, string ip, int port, int num)
        {
            string documentDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string LoadPath = Path.Combine(documentDir, file);
            int size = 0;
            int err = 0;

            //Loading layout files.
            bool sts = api.LoadSopFile(LoadPath, ref size);

            //Successful read?
            if ((sts == true) && (size > 0))
            {
                //print.
                sts = api.WLANPrintSopFile(ip, port, num);
                //print error?
                if (sts == false)
                {
                    //Set error cause.
                    err = api.ResultErrorCode;
                }
                //Writing layout files.
                sts = api.SaveSopFile(LoadPath);
                //writing error?
                if (sts == false)
                {
                    //Set error cause.
                    err = api.ResultErrorCode;
                }
            }
            //Read failed.
            else
            {
                //Set error cause.
                err = api.ResultErrorCode;
            }
            return err;
        }

        //Create a layout file.
        public int MakeFile(string file)
        {
            string documentDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string SavePath = Path.Combine(documentDir, file);
            string ImagePath01 = Path.Combine(documentDir, @"01.png");
            string ImagePath02 = Path.Combine(documentDir, @"02.png");
            string PreviewPath = Path.Combine(documentDir, @"preview.png");
            string item_name = @"";
            System.Drawing.Point st = new System.Drawing.Point();
            System.Drawing.Point ed = new System.Drawing.Point();
            System.Drawing.Rectangle rect = new System.Drawing.Rectangle();
            int err = 0;

            //Create New.
            bool sts = api.NewSopFile(LayoutDesignerSDK.LibApi.PRINTER_SM4_21, 48 * 8, 70 * 8, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Serial counter initialization.
            sts = api.InitSerial();
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Table frame.
            rect.X = 45;
            rect.Y = 48;
            rect.Width = 275;
            rect.Height = 255;
            item_name = @"";
            sts = api.SetRectData(ref item_name, rect, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 44;
            st.Y = 80;
            ed.X = 319;
            ed.Y = 80;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 44;
            st.Y = 160;
            ed.X = 319;
            ed.Y = 160;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 44;
            st.Y = 192;
            ed.X = 319;
            ed.Y = 192;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 44;
            st.Y = 224;
            ed.X = 319;
            ed.Y = 224;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 44;
            st.Y = 263;
            ed.X = 319;
            ed.Y = 263;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Frame border.
            st.X = 144;
            st.Y = 47;
            ed.X = 144;
            ed.Y = 263;
            item_name = @"";
            sts = api.SetLineData(ref item_name, (System.Drawing.Point)st, (System.Drawing.Point)ed, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Sencha - "煎茶"
            rect.X = 157;
            rect.Y = 22;
            rect.Width = 52;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"煎茶", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Name - "名称"
            rect.X = 49;
            rect.Y = 55;
            rect.Width = 95;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"名　　称", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Sencha - 煎茶
            rect.X = 200;
            rect.Y = 55;
            rect.Width = 52;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"煎茶", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Raw material name - "原材料名"
            rect.X = 47;
            rect.Y = 110;
            rect.Width = 95;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"原材料名", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Tea - "茶（埼玉県産）"
            rect.X = 151;
            rect.Y = 111;
            rect.Width = 161;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"茶（埼玉県産）", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Internal capacity - "内容量"
            rect.X = 47;
            rect.Y = 166;
            rect.Width = 95;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"内 容 量", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //60g
            rect.X = 207;
            rect.Y = 165;
            rect.Width = 41;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"60g", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Expiry date - "賞味期限"
            rect.X = 47;
            rect.Y = 199;
            rect.Width = 95;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"賞味期限", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //August 2022 - "2022年8月"
            rect.X = 181;
            rect.Y = 198;
            rect.Width = 106;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"2022年8月", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Preservation method - "保存方法"
            rect.X = 47;
            rect.Y = 232;
            rect.Width = 95;
            rect.Height = 23;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"保存方法", 0, @"Hiragino Sans", 16, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Please avoid high temperature and humidity and be careful of lingering incense - "高温多湿を避け移り香にご注意下さい"
            rect.X = 144;
            rect.Y = 226;
            rect.Width = 180;
            rect.Height = 39;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, "高温多湿を避け移り\n香にご注意下さい", 0, @"Hiragino Sans", 14, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Manufacturer - "製造者：株式会社狭山煎茶東京都豊島区池袋2-51-13"
            rect.X = 48;
            rect.Y = 264;
            rect.Width = 347;
            rect.Height = 39;
            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, "製造者：株式会社狭山煎茶\n東京都豊島区池袋2-51-13", 0, @"Hiragino Sans", 14, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Handling Precautions - "＜取扱上の注意＞"
            rect.X = 41;
            rect.Y = 309;
            rect.Width = 225;
            rect.Height = 41;

            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, "＜取扱上の注意＞\nお茶は鮮度が大切です。\n開封後はお早めにお飲みください。", 0, @"Hiragino Sans", 10, 0, 0, 0, 0, 0, 0, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Set a 1D barcode.
            rect.X = 40;
            rect.Y = 361;
            rect.Width = 230;
            rect.Height = 48;
            item_name = @"";
            sts = api.SetBarcode1dData(ref item_name, rect, @"123456789012", 0, LayoutDesignerSDK.DefineData.BARCODE1D_TYPE_JAN13, 2, 0, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Paper logo image.
            rect.X = 267;
            rect.Y = 305;
            rect.Width = 50;
            rect.Height = 50;
            item_name = @"";
            sts = api.SetImageData(ref item_name, rect, ImagePath01, 0, 128, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Plastic logo image.
            rect.X = 267;
            rect.Y = 377;
            rect.Width = 50;
            rect.Height = 50;
            item_name = @"";
            sts = api.SetImageData(ref item_name, rect, ImagePath02, 0, 128, true, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Exterior - "外装"
            rect.X = 256;
            rect.Y = 353;
            rect.Width = 66;
            rect.Height = 25;

            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, "外装\n外装ラベル", 0, @"Hiragino Sans", 9, 0, 0, 0, 0, 0, 1, false, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Interior - "内装"
            rect.X = 276;
            rect.Y = 425;
            rect.Width = 29;
            rect.Height = 13;

            item_name = @"";
            sts = api.SetTextData(ref item_name, rect, @"内装", 0, @"Hiragino Sans", 9, 0, 0, 0, 0, 0, 1, false, 0);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //A set of document macros.
            string[] start = { @"PRT_DENSITY,120" };
            string[] end = { @"FEED,120" };
            item_name = @"";

            sts = api.SetMacroData(ref item_name, start, end);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Serial number initialization.
            api.InitSerial();

            //Save layout file.
            sts = api.SaveSopFile(SavePath);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Create a preview image.
            sts = api.PreviewSopFile(PreviewPath);
            //Error?
            if (sts == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            return err;
        }

        //Create a layout file.
        public int RegistMacro(string ip, int port, string file)
        {
            string documentDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string LoadPath = Path.Combine(documentDir, file);
            string SavePath = Path.Combine(documentDir, @"standalone_macro.dat");
            bool ret;
            int err = 0;
            int macro_size = 0;
            int count = 0;
            int size = 0;

            //standalone Macro Command
            string[] stand_cmd_init = { "PRT_INIT,0,0,0,0,0,0,,,1" };
            string[] stand_cmd_cr17 = { "CR,0,0,0,0,0,0,,,17" };
            string[] stand_cmd_cr5 = { "CR,0,0,0,0,0,0,,,5" };
            string[] stand_cmd_time0 = { "TIME_PRINT,0,0,0,0,0,0,,,1" };
            string[] stand_cmd_qrcode = { "QR_CODE,8,2,0,0,0,0,SANEI ELEC,,1" };
            string[] stand_cmd_counter = { "COUNTER_PRINT,0,0,0,0,0,0,,,1" };
            string[] stand_cmd_alignment = { "ALIGNMENT,1,0,0,0,0,0,,,1" };

            //SOP file load.
            ret = api.LoadSopFile(LoadPath,ref size);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Initialize a standalone macro.
            api.InitStandaloneMacro();

            //Date & Time
            api.SetStandaloneMacroTitle(0, "1 Date & Time");
            ret = api.SetStandaloneMacroData(0, stand_cmd_init);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(0, stand_cmd_cr17);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(0, stand_cmd_alignment);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(0, stand_cmd_time0);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(0, stand_cmd_cr5);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Counter
            api.SetStandaloneMacroTitle(1, "2 Counter");
            ret = api.SetStandaloneMacroData(1, stand_cmd_init);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(1, stand_cmd_cr17);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(1, stand_cmd_alignment);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(1, stand_cmd_counter);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(1, stand_cmd_cr5);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //QR code
            api.SetStandaloneMacroTitle(2, "3 QR code");
            ret = api.SetStandaloneMacroData(2, stand_cmd_init);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(2, stand_cmd_cr17);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(2, stand_cmd_alignment);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(2, stand_cmd_qrcode);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }
            ret = api.SetStandaloneMacroData(2, stand_cmd_cr5);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Saving a standalone macro file.
            ret = api.SaveStandaloneMacro(SavePath);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Loading a standalone macro file.
            ret = api.LoadStandaloneMacro(SavePath, ref macro_size);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Get the command set count for a standalone macro.
            ret = api.GetStandaloneMacroCount(0, ref count);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Get command string area.
            string[] work = new string[count];
            ret = api.GetStandaloneMacroData(0, ref work);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            //Standalone Macro Registration.
            ret = api.RegistStandaloneMacro(ip, port, 0);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            ret = api.RegistStandaloneMacro(ip, port, 1);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            ret = api.RegistStandaloneMacro(ip, port, 2);
            if (ret == false)
            {
                //Set error cause.
                err = api.ResultErrorCode;
                return err;
            }

            return 0;
        }

        //View error alerts.
        public void ErrorMessage(string error, int code)
        {
            var alert = UIAlertController.Create(@"Error", error + "\nerror code = " + code.ToString(), UIAlertControllerStyle.Alert);
            alert.AddAction(UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null));
            UIApplication.SharedApplication.KeyWindow.RootViewController.PresentViewController(alert, true, null);
        }

        //Return the version.
        public string Version()
        {
            return SdkVersion;
        }
    }
}

